/*************************************************************************
  Copyright (c) 1993 - 1997
  Digital Equipment Corporation, Maynard, Mass.

  This software is furnished under a license and may be used and  copied
  only  in  accordance  with  the  terms  of  such  license and with the
  inclusion of the above copyright notice.  This software or  any  other
  copies  thereof may not be provided or otherwise made available to any
  other person.  No title to and ownership of  the  software  is  hereby
  transferred.

  The information in this software is subject to change  without  notice
  and  should  not  be  construed  as  a commitment by DIGITAL Equipment
  Corporation.
 *************************************************************************/


/*++
* ==========
* = xcbr.c =
* ==========
*
* Description:
*   This application demonstrates the API's ability to create and open a PVC
*   using CBR.  This application builds an internal packet and repeatedly
*   transmits it over the PVC.  Should the send fail, the application notifies
*   the user.  Otherwise a "." is printed on success.  This application is used
*   in conjunction with the api_rcv_cbr executable compiled from rcbr.c.
*
* Authors:
*   TLR - Theodore L. Ross
*   PAE - Paul A. Ely
*
* Modification History:
*   Date          Name     Description
*   22-Sep-1996   TLR      Created.
*   07-Jul-1997   PAE      Modified application to allow non-zero vpi values.
*   21-Jul-1997   PAE      Modified application to allow user-specifiable
*                          packet sizes to 64K bytes.
*
* Arguments:
*   driver name   - The driver service name, e.g. lpa0 or lta0.
*   vpi           - The VPI to use in the PVC.
*   vci           - The VCI to use in the PVC.
*   pkt_size      - The packet size in bytes.  Bound by 0<pkt_size<65536.
*   kbps          - The constant bit rate desired in Kbits/sec.
*   window        - Framing size, typically 0.
*
* Issues:
*   none.
--*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/errno.h>
#include <unistd.h>
#include <netaal/aal.h>

#define SIZE  65535
#define errtext(n) n

void main (int argc, char **argv)
   {
   int                  sock;
   int                  a, i, window, bit_rate;
   int                  nowindow = 0;
   unsigned int         cell_rate;
   unsigned int         value, pkt_size;
   unsigned int         device_cnt;
   unsigned char        buff[65535];
   char                 drv_name[32];
   ATM_CONNECTION_ID    conn_id;
   ATM_PARAMS           params;
   ATM_DEVICE_INFO      device_info;

   /* The user must pass six command-line arguments. */

   if (argc != 7)
      {
      printf ("Usage: api_xmt_cbr drv_name vpi# vci# pkt_size kbit_rate 0\n");
      printf ("\n");
      printf ("   Where drv_name is either lpa or lta and its instance value\n");
      printf ("         vpi# is the vpi of interest\n");
      printf ("         vci# is the vci of interest\n");
      printf ("         pkt_size is the packet size in bytes, 0<pkt_size<65536\n");
      printf ("         kbit_rate is the ((bits/sec) / 1000) desired\n");
      printf ("         0 is the default.  Argument reserved. \n");
      exit(1);
      }

   /* Clear out the driver name array. */

   for (i = 0; i < 32; i++)
      drv_name[i] = '\0';

   /* Set the local 64K buffer to a hexadecimal incrementing pattern. */

   value = 1;
   for (i = 0; i < SIZE; i++)
      {
      buff[i] = value;
      value += 1;
      if (value > 0xF) value = 1;
      }

   /* Only lpa and lta driver names are currently supported. */
 
   if (strncmp (argv[1], "lpa", 3) &&
       strncmp (argv[1], "lta", 3) != 0)
      {
      printf ("Unsupported driver service name\n");
      exit (-1);
      }

   /* Copy the driver service instance into the conn_id structure. */
   
   strncpy (&drv_name[0], argv[1], (strlen (argv[1]) + 1));

   for (i = 0; i < strlen (&drv_name[0]); i++)
      {
      if ((drv_name[i] >= '0') && (drv_name[i] <= '9'))
         {
         sscanf (&drv_name[i], "%d", &conn_id.device_number);
         break;
         }
      }

   /* Scan the user's command-line parameters and convert into the correct
   *    type.
   */

   sscanf (argv[2], "%d", &conn_id.vpi);
   sscanf (argv[3], "%d", &conn_id.vci);
   sscanf (argv[4], "%d", &pkt_size);
   sscanf (argv[5], "%d", &bit_rate);
   sscanf (argv[6], "%d", &window);

   /* The window argument is used as a signaling event.  If the user passes
   *    the default value 0, prevent the application from waiting for a signal
   *    from the other end.
   */

   if (window == 0)
      {
      window   = 1;
      nowindow = 1;
      }

   /* Open an AAL socket for DGRAM service. */

   printf ("[ Creating ATM Socket ]\n");
   sock = socket (AF_AAL, SOCK_DGRAM, 0); 
   if (sock == -1)
      {
      printf ("[ socket call failed, error: %d ]\n", errtext (errno));
      exit (1);
      }

   /* Get the driver's hard mtu size.  This value is required for establishing
   *    the fmtu and bmtu for the PVC.
   */

   device_info.device_number   = conn_id.device_number;
   device_info.driver_name     = drv_name;
   device_info.driver_name_len = 32;
   if (ioctl (sock, SIO_GET_DEVICE_INFO, &device_info) == -1)
      {
      printf ("Device %d inaccessible, socket error: %d\n",
              device_info.device_number,
              errno);
      }

   /* Set the params structure to its defaults.  Then, set the params structure
   *    to the known values for the CBR service.  Note the following calculation
   *    for the forward and backward peak cell rates (cells/sec).
   *
   *    C/S = BitRate / ((BitsPerByte) * (BytesPerCell))
   *    Note that (8 bits/sec * 53 bytes/cell) = 424 bits/cell
   */

   cell_rate            = (bit_rate * 1000) / 424;

   ATM_DEFAULTS (&params);
   params.fmtu          = device_info.hard_mtu;
   params.bmtu          = device_info.hard_mtu;
   params.fpeakcr[1]    = cell_rate;
   params.bpeakcr[1]    = cell_rate;
   params.fqos          = ATM_QC_CLASSA;
   params.bqos          = ATM_QC_CLASSA;
   params.flags         = ATM_FLAG_CBR;

   /* Write the new parameters down to the driver. */

   if (ioctl (sock, SIO_SET_ATM_PARAMETERS, &params) == -1)
      {
      printf ("[ param ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Associate the socket with the PVC VCI. */

   printf ("[ Associating Socket with PVC: %s/%d/%d ]\n",
	         &drv_name[0],
            conn_id.vpi,
            conn_id.vci);

   if (ioctl (sock, SIO_ASSOCIATE_PVC, &conn_id) == -1)
      {
      printf ("[ socket ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Continuously send the buffer through the socket interface. */

   while (1)
      {
      for (a = 0; a < window; a++)
         {
         i = send (sock, buff, pkt_size, 0);
         if (i == -1)
            {
            a--;
            if (errno != ENOBUFS) printf ("xmt fail - err = %d\n", errno);
            }
         else
            {
            printf (".");
            fflush (stdout);
            }
         }

      if (!nowindow)
         while (recv (sock, buff, 1, 0) < 1);
      }

   /* Application done.  Close socket resources and exit. */

   close (sock);
   return;
   }

